"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transferSols = exports.transferSpl = exports.SolanaSwapClient = void 0;
const web3_js_1 = require("@solana/web3.js");
const spl_token_1 = require("@solana/spl-token");
const RaydiumSwap_1 = __importDefault(require("../server/RaydiumSwap"));
const utils_1 = require("../utils");
const helpers_1 = require("@solana-developers/helpers");
class SolanaSwapClient {
    constructor(poolKeys, devnet = false) {
        this.rpc = 'https://api.mainnet-beta.solana.com';
        this.rpc = devnet ? 'https://api.devnet.solana.com' : 'https://api.mainnet-beta.solana.com';
        this.connection = new web3_js_1.Connection(this.rpc);
        if (!poolKeys || poolKeys.length === 0) {
            throw new Error('Pool keys not loaded');
        }
        this.poolKeys = poolKeys;
        if (!this.poolKeys || this.poolKeys.length === 0) {
            throw new Error('Pool keys not loaded');
        }
    }
    swap(fromPrivKeyBs58_1, tokenIn_1, tokenOut_1, amountIn_1) {
        return __awaiter(this, arguments, void 0, function* (fromPrivKeyBs58, tokenIn, tokenOut, amountIn, slippage = "5", // 5%
        priceImpact = null, speedUp = false, maxRetries = 20) {
            var _a, _b, _c, _d, _e, _f, _g, _h, _j;
            const owner = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
            const amountInSols = (0, utils_1.convertNumber)(amountIn);
            const raydiumSwap = new RaydiumSwap_1.default(this.rpc, owner, this.poolKeys);
            const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenIn, tokenOut);
            if (!poolInfo) {
                throw new Error("Pool info not found");
            }
            const transaction = yield raydiumSwap.getSwapTransaction(tokenOut, amountInSols, poolInfo, slippage, priceImpact, speedUp);
            const txid = yield raydiumSwap.sendLegacyTransaction(transaction, maxRetries);
            const latestBlockhash = yield this.connection.getLatestBlockhash();
            yield this.connection.confirmTransaction({
                signature: txid,
                blockhash: latestBlockhash.blockhash,
                lastValidBlockHeight: latestBlockhash.lastValidBlockHeight,
            }, "confirmed");
            const tx = yield this.connection.getTransaction(txid, { "commitment": "confirmed", "maxSupportedTransactionVersion": 1 });
            let amountOut = 0;
            if (tokenOut === "So11111111111111111111111111111111111111112") {
                const preBalance = (_a = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _a === void 0 ? void 0 : _a.preBalances[0];
                const postBalance = (_b = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _b === void 0 ? void 0 : _b.postBalances[0];
                const fee = (_c = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _c === void 0 ? void 0 : _c.fee;
                const amountOutLamport = (postBalance !== null && postBalance !== void 0 ? postBalance : 0) - ((preBalance !== null && preBalance !== void 0 ? preBalance : 0) - (fee !== null && fee !== void 0 ? fee : 0));
                amountOut = parseFloat(amountOutLamport.toString()) / web3_js_1.LAMPORTS_PER_SOL;
            }
            else {
                const preTokenOutBalance = (_e = (_d = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _d === void 0 ? void 0 : _d.preTokenBalances) === null || _e === void 0 ? void 0 : _e.find(obj => obj.mint === tokenOut && obj.owner === owner.publicKey.toBase58());
                const postTokenOutBalance = (_g = (_f = tx === null || tx === void 0 ? void 0 : tx.meta) === null || _f === void 0 ? void 0 : _f.postTokenBalances) === null || _g === void 0 ? void 0 : _g.find(obj => obj.mint === tokenOut && obj.owner === owner.publicKey.toBase58());
                amountOut = ((_h = postTokenOutBalance === null || postTokenOutBalance === void 0 ? void 0 : postTokenOutBalance.uiTokenAmount.uiAmount) !== null && _h !== void 0 ? _h : 0) - ((_j = preTokenOutBalance === null || preTokenOutBalance === void 0 ? void 0 : preTokenOutBalance.uiTokenAmount.uiAmount) !== null && _j !== void 0 ? _j : 0);
            }
            return {
                txid,
                amountOut
            };
        });
    }
    calcAmountOut(fromPrivKeyBs58_1, tokenIn_1, tokenOut_1, amountIn_1) {
        return __awaiter(this, arguments, void 0, function* (fromPrivKeyBs58, tokenIn, tokenOut, amountIn, slippage = "5", // 5%
        priceImpact = null) {
            const owner = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
            const amountInSols = (0, utils_1.convertNumber)(amountIn);
            const raydiumSwap = new RaydiumSwap_1.default(this.rpc, owner, this.poolKeys);
            const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenIn, tokenOut);
            if (!poolInfo) {
                throw new Error("Pool info not found");
            }
            const directionIn = poolInfo.quoteMint.toString() == tokenOut;
            const amountOut = yield raydiumSwap.calcAmountOut(poolInfo, amountInSols, directionIn, slippage, priceImpact);
            const amountOutParsed = amountOut.amountOut.raw.toNumber() /
                Math.pow(10, amountOut.amountOut.currency.decimals);
            return {
                amountIn: amountInSols,
                amountOut: amountOutParsed,
            };
        });
    }
}
exports.SolanaSwapClient = SolanaSwapClient;
function transferSpl(connection_1, fromPrivKeyBs58_1, token_1, amount_1, receiver_1) {
    return __awaiter(this, arguments, void 0, function* (connection, fromPrivKeyBs58, token, amount, receiver, speedUp = false) {
        const to = (0, utils_1.convertPubkey)(receiver);
        const from = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
        const amountInSols = (0, utils_1.convertNumber)(amount);
        const fromAssociatedTokenAccount = yield (0, spl_token_1.getAssociatedTokenAddress)((0, utils_1.convertPubkey)(token), from.publicKey);
        const toAssociatedTokenAccount = yield (0, spl_token_1.getAssociatedTokenAddress)((0, utils_1.convertPubkey)(token), to);
        const tx = new web3_js_1.Transaction();
        try {
            const toAccountInfo = yield connection.getAccountInfo(toAssociatedTokenAccount);
            if (!toAccountInfo) {
                tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(from.publicKey, toAssociatedTokenAccount, to, (0, utils_1.convertPubkey)(token)));
            }
        }
        catch (e) {
            tx.add((0, spl_token_1.createAssociatedTokenAccountInstruction)(from.publicKey, toAssociatedTokenAccount, to, (0, utils_1.convertPubkey)(token)));
        }
        tx.add((0, spl_token_1.createTransferInstruction)(fromAssociatedTokenAccount, toAssociatedTokenAccount, from.publicKey, amountInSols * web3_js_1.LAMPORTS_PER_SOL));
        if (speedUp) {
            const priorityFees = yield connection.getRecentPrioritizationFees({ lockedWritableAccounts: [from.publicKey] });
            const totalFees = priorityFees.reduce((sum, item) => sum + item.prioritizationFee, 0);
            const averageFee = Math.floor((Math.floor(totalFees / priorityFees.length) * 1.2));
            tx.instructions.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
                microLamports: averageFee
            }));
            const units = yield (0, helpers_1.getSimulationComputeUnits)(connection, tx.instructions, from.publicKey, []);
            if (units) {
                tx.instructions.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitLimit({
                    units
                }));
            }
        }
        tx.recentBlockhash = (yield connection.getLatestBlockhash()).blockhash;
        tx.feePayer = from.publicKey;
        const recoverTx = web3_js_1.Transaction.from(tx.serialize({ requireAllSignatures: false }));
        // transaction sign
        recoverTx.sign(from);
        const signature = yield connection.sendRawTransaction(recoverTx.serialize());
        return signature;
    });
}
exports.transferSpl = transferSpl;
function transferSols(connection_1, fromPrivKeyBs58_1, amount_1, receiver_1) {
    return __awaiter(this, arguments, void 0, function* (connection, fromPrivKeyBs58, amount, receiver, speedUp = false) {
        const to = (0, utils_1.convertPubkey)(receiver);
        const from = (0, utils_1.convertKeypair)(fromPrivKeyBs58);
        const amountInSols = (0, utils_1.convertNumber)(amount);
        const transferInstruction = [
            web3_js_1.SystemProgram.transfer({
                fromPubkey: from.publicKey,
                toPubkey: to,
                lamports: amountInSols * web3_js_1.LAMPORTS_PER_SOL
            }),
        ];
        if (speedUp) {
            const priorityFees = yield connection.getRecentPrioritizationFees();
            const totalFees = priorityFees.reduce((sum, item) => sum + item.prioritizationFee, 0);
            const averageFee = Math.floor((Math.floor(totalFees / priorityFees.length) * 1.2));
            transferInstruction.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitPrice({
                microLamports: averageFee
            }));
            const units = yield (0, helpers_1.getSimulationComputeUnits)(connection, transferInstruction, from.publicKey, []);
            if (units) {
                transferInstruction.unshift(web3_js_1.ComputeBudgetProgram.setComputeUnitLimit({
                    units
                }));
            }
        }
        let { blockhash } = yield connection.getLatestBlockhash();
        const message = new web3_js_1.TransactionMessage({
            payerKey: from.publicKey,
            recentBlockhash: blockhash,
            instructions: transferInstruction,
        }).compileToV0Message();
        const transaction = new web3_js_1.VersionedTransaction(message);
        transaction.sign([from]);
        const signature = yield connection.sendRawTransaction(transaction.serialize());
        return signature;
    });
}
exports.transferSols = transferSols;
