import RaydiumSwap from "./RaydiumSwap";
import { Transaction } from "@solana/web3.js";
import "dotenv/config";

export const swap = async ({
  raydiumSwap,
  tokenA,
  tokenB,
  amountIn,
  direction,
  maxRetries,
}: {
  raydiumSwap: RaydiumSwap;
  tokenA: string;
  tokenB: string;
  amountIn: number;
  direction: "AtoB" | "BtoA";
  maxRetries: number;
}): Promise<string> => {
  const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenA, tokenB);
  if (!poolInfo) {
    throw new Error("Pool info not found");
  }
  const toToken = direction === "AtoB" ? tokenB : tokenA;
  const transaction = await raydiumSwap.getSwapTransaction(
    toToken,
    amountIn,
    poolInfo
  );

  const txid = await raydiumSwap.sendLegacyTransaction(
    transaction as Transaction,
    maxRetries
  );

  return txid;
};

export const calcAmountOut = async ({
  raydiumSwap,
  tokenA,
  tokenB,
  amountIn,
  direction,
}: {
  raydiumSwap: RaydiumSwap;
  tokenA: string;
  tokenB: string;
  amountIn: number;
  direction: "AtoB" | "BtoA";
}): Promise<{
  amountIn: number;
  amountOut: number;
}> => {
  const poolInfo = raydiumSwap.findPoolInfoForTokens(tokenA, tokenB);
  if (!poolInfo) {
    throw new Error("Pool info not found");
  }

  const amountOut = await raydiumSwap.calcAmountOut(
    poolInfo,
    amountIn,
    direction === "AtoB"
  );

  const amountOutParsed =
    amountOut.amountOut.raw.toNumber() /
    10 ** amountOut.amountOut.currency.decimals;

  return {
    amountIn,
    amountOut: amountOutParsed,
  };
};
